////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* APL / FAHREN  		              */
/* Version 1.0.0                      */
/* 2011-06-07                         */
/*                                    */
/**************************************/
////////////////////////////////////////



/***************   Fahren   ***************/

Fahren:

	rcall Fahren_Start	; Fahren ohne Linienverfolgung fr 500 ms
	rcall LINEFOLLOW	; Linienverfolgung bis Ziel gefunden
	cbi   ADCSRA,ADIE   ; ADC Interrupt abschalten
	// Timer abschalten
	in   R16,TIMSK
	andi R16,0b11111110
	out  TIMSK,R16
ret



/***************   FORWARD   ***************/

Fahren_Start:

	push  R16					; Register sichern
	push  R17
	push  R18

	ldi R18,messwert
	tst R18
	brne Fahren_Start_2

	rcall MotorDir_FWD_FWD		; Vorwrtsfahrt initierien
	ldi   R16,speed				; Standardgeschwindigkeit einstellen
	mov   sr,R16
	mov   sl,R16
	rcall MotorSpeed			; Motoren auf Geschwindigkeit einstellen
	clr   R16					; Wartezeit initialisieren
	clr   R17
	ldi   R18,0x07

	Fahren_Start_1:

	dec   R16					; Wartezeit damit der ASURO vom Licht (Zielpunkt) sich 
	brne  Fahren_Start_1		; entfernt hat und nicht gleich wieder stopt.
	dec   R17					
	brne  Fahren_Start_1		
	dec   R18					
	brne  Fahren_Start_1		
	rcall Motor_Stop

	Fahren_start_2:

	pop	  R18					; Register wiederherstellen
	pop   R17
	pop   R16

ret



/***************   LINEFOLLOW   ***************/

LINEFOLLOW:

	push  R18				; Register sichern
	push  R19
	push  ZH
	push  ZL

	rcall LINEFOLLOW_INIT	; Linienverfolgung initialisieren

	LINEFOLLOW_1:

	tst mr					; Prfen ob rechter Sensor Ziel erkannt hat
	brne LINEFOLLOW_2
	tst ml					; Prfen ob linker Sensor Ziel erkannt hat
	brne LINEFOLLOW_2
	rcall Motor_Stop		; Wurde Ziel von beiden Sensoren erkannt, dann ASURO anhalten
	rjmp  LINEFOLLOW_ENDE	; Nachdem Ziel erreicht wurde - Linienverfolgung beenden

	LINEFOLLOW_2:

	tst  diff				; Prfen ob Differenz negativ oder postiv ist, entsprechend gegensteuern
	brmi rechts_fahren		; bei negativen Wert rechts fahren
	rjmp links_fahren		; bei positiven Wert links fahren	

	rechts_fahren:

	ser   R18				; negativen Differenzwert in positiven Wert wandeln (255 - Wert + 1)
	sub   R18,diff
	inc   R18
	cpi   R18,7				; Prfen ob Differenz grer als 7 ist
	brcs  rechts_fahren_1	; wenn nicht dann Rechtsfahren initieren entsprechend Reglerwert
	ldi   R18,7				; wenn grer als 7, dann auf maximalen Reglerwert einstellen

	rechts_fahren_1:

	ldi   ZH,high(pdr1)		; Geschwindigkeitswert fr das Rad in nicht abtrifftender Richtung 
	ldi   ZL,low(pdr1)		; aus Tabelle fr Regler 1 laden
	clr   R19
	add   ZL,R18
	adc   ZH,R19
	ld    R19,Z
	mov   sr,R19
	ldi   ZH,high(pdr2)		; Geschwindigkeit fr das in Drifftrichtung liegende Rad aus Tabelle
	ldi   ZL,low(pdr2)		; fr Regler 2 laden
	clr   R19
	add   ZL,R18
	adc   ZH,R19
	ld    R19,Z
	mov   sl,R19
	rcall MotorSpeed		; Geschwindigkeiten einstellen
	rjmp  LINEFOLLOW_1

	links_fahren:

	mov   R18,diff			; Prfen ob Differenz grer als 7 ist
	cpi   R18,7				; wenn nicht dann Rechtsfahren initieren entsprechend Reglerwert
	brcs  links_fahren_1	; wenn grer als 7, dann auf maximalen Reglerwert einstellen
	ldi   R18,7

	links_fahren_1:

	ldi   ZH,high(PDR1)		; Geschwindigkeitswert fr das Rad in nicht abtrifftender Richtung
	ldi   ZL,low(PDR1)		; aus Tabelle fr Regler 1 laden
	clr   R19
	add   ZL,R18
	adc   ZH,R19
	ld    R19,Z
	mov   sl,R19
	ldi   ZH,high(PDR2)		; Geschwindigkeit fr das in Drifftrichtung liegende Rad aus Tabelle
	ldi   ZL,low(PDR2)		; fr Regler 2 laden
	clr   R19
	add   ZL,R18
	adc   ZH,R19
	ld    R19,Z
	mov   sr,R19
	rcall MotorSpeed		; Geschwindigkeiten einstellen
	rjmp LINEFOLLOW_1

	LINEFOLLOW_ENDE:	

	pop   ZL			; Register wiederherstellen	
	pop   ZH
	pop   R19
	pop   R18

ret



/***************   LINEFOLLOW_INIT   *****************/

LINEFOLLOW_INIT:

	//	Register sichern
	push R18
	push ZH
	push ZL 

	//	Tabelle Regler 1 anlegen
	ldi ZH,high(pdr1)
	ldi ZL,low(pdr1)
	ldi R18,160
	st  Z+,R18
	ldi R18,105
	st  Z+,R18
	ldi R18,72
	st  Z+,R18
	ldi R18,51
	st  Z+,R18
	ldi R18,39
	st  Z+,R18
	ldi R18,31
	st  Z+,R18
	ldi R18,27
	st  Z+,R18
	ldi R18,20
	st  Z+,R18

	//	Tabelle Regler 2 anlegen
	ldi ZH,high(pdr2)
	ldi ZL,low(PDR2)
	ldi R18,160
	st  Z+,R18
	ldi R18,125
	st  Z+,R18
	ldi R18,106
	st  Z+,R18
	ldi R18,96
	st  Z+,R18
	ldi R18,91
	st  Z+,R18
	ldi R18,88
	st  Z+,R18
	ldi R18,87
	st  Z+,R18
	ldi R18,85
	st  Z+,R18

	//	Startwerte setzen
	ldi R18,0x01
	mov mr,R18
	mov ml,R18

	//	Timer starten
	ldi R18,wartezeit
	out TCNT0,R18
	ldi R18,(1 << TOIE0)
	out TIMSK,R18
	
	//	Register wiederherstellen
	pop ZL
	pop ZH
	pop R18

ret


/***************   MotorDir_FWD_FWD   ***************/

MotorDir_FWD_FWD:

	sbi PORTD,5
	cbi PORTD,4
	sbi PORTB,5
	cbi PORTB,4

ret



/***************   MotorSpeed   ***************/

MotorSpeed:

	push  R16

	ldi R16,messwert
	tst R16
	brne Motor_Speed_TX

	clr   R16
	out   OCR1AH, R16
	out   OCR1AL, sl
	out   OCR1BH, R16
	out   OCR1BL, sr
	rjmp  Motor_Speed_End

	Motor_Speed_TX:

	rcall TRANSMIT_ENABLE
	push R25
	TXI 'L'
	TXI ':'
	TX_BIN2ASCII R25,ml
	TXI 'R'
	TXI ':'
	TX_BIN2ASCII R25,mr
	TXI 'D'
	TXI ':'
	TX_BIN2ASCII R25,diff
	TXI 'S'
	TXI 'L'
	TXI ':'
	TX_BIN2ASCII R25,sl
	TXI 'S'
	TXI 'R'
	TXI ':'
	TX_BIN2ASCII R25,sr
	TXI 10
	TXI 13
	pop R25

	Motor_Speed_End:	

	pop   R16


ret



/****************   Motor_Stop   ***************/

Motor_Stop:

	push R16
	clr  R16
	out  OCR1AH, R16
	out  OCR1AL, R16
	out  OCR1BH, R16
	out  OCR1BL, R16
	mov  sr,R16
	mov  sl,R16
	pop  R16

ret
