////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* APL / ISR    		              */
/* Version 1.0.0                      */
/* 2011-06-07                         */
/*                                    */
/**************************************/
////////////////////////////////////////



/***************   TIM0_OVF   ***************/
/*
TIM0_OVF:
	
	push R16			; Flags sichern
	in   R16,SREG
	push R16
	
	dec  t_count		; Zhler dekrementieren
	brne TIM0_OVF_1		; wenn Zhler noch nicht 0 dann weiter bei TIM0_OVF_1
	ser  R16			; Register 11 auf 0xFF setzen zur Anzeige, dass ein Timeout vorliegt
	mov  timeout,R16		

TIM0_OVF_1:

	pop  R16
	out  SREG,R16
	pop  R16

reti
*/



/*************** Timer   ***************/

TIM0_OVF:

	push R16
	in   R16,SREG
	push R16
	
	// Timer abschalten
	in   R16,TIMSK
	andi R16,0b11111110
	out  TIMSK,R16

	// ADC einschalten
	sbi ADCSRA,ADIE
	sbi ADCSRA,ADSC

	pop R16
	out SREG,R16
	pop R16

reti



/***************   ADCI   ****************/

adci:

	push R16		; Register sichern
	in   R16,SREG
	push R16
	push R18
	push R28

	adci_0:	
	
	in   R28,ADCH	; Messergebnis auslesen
	mov  R18,adcc	; Zhler kopieren
	lsr  R18		; auslesen des ersten Bits
	brcc adci_1		

	//	Wenn erstes Bit des Zhlers gesetzt ist, dann Messwert linker Sensor mit LED
	//	zwischenspeichern und ADC auf Messung linker Sensor ohne LED vorbereiten

	lsl  adcc		; ADC Zhler setzen fr zweite Messung
	mov  zel,R28	; Messwert in Zwischenspeicher (zel - Zwischenergebnis links) speichern
	cbi  PORTD,6	; Front LED abschalten
	rjmp adci_4

	adci_1:

	lsr  R18		; auslesen zweites Bit des Zhlers
	brcc adci_2		

	//	Wenn zweites Bit gesetzt ist, dann Messwert linker Sensor ohne LED vom
	//	Zwischengespeicherten Messwert mit LED abziehen und zwischenspeichern
	//	sowie ADC fr Messung rechten Sensor mit LED vorbereiten
	
	lsl  adcc		; ADC Zhler setzen fr dritte Messung
	sub  zel,R28	; Messwert von Zwischenergebnis abziehen
	ldi  R16, (1 << REFS0)|(1 << ADLAR)|(1 << MUX1)|(1 << MUX0)	; ADC auf rechten Sensor einstellen
	out  ADMUX,R16
	sbi  PORTD,6	; Front LED einschalten
	rjmp adci_4

	adci_2:

	lsr  R18		; auslesen drittes Bit des Zhlers
	brcc adci_3

	//	Wenn drittes Bit des Zhlers gesetzt ist, dann Messwert rechter Sensor mit LED zwischenspeichern
	//	und ADC auf Messung rechter Sensor ohne LED vorbereiten

	lsl  adcc		; ADC Zhler setzen fr vierte Messung
	mov  zer,R28	; Messwert in Zwischenspeicher (zer - Zwischenergebnis rechts) speichern
	cbi  PORTD,6	; Front LED abschalten
	rjmp adci_4

	adci_3:

	//	Wenn auch drittes Bit nicht gesetzt war, dann wurde vierte Messung durchgefhrt.
	//	Dazu Messwert rechter Sensor ohne LED von Zwischenergebnis Messung rechter Sensor mit LED abziehen 
	//	und zwischenspeichern, anschlieend Auswertung aller Zwischenergebnisse und in den
	//	entsprechenden Registern die Ergebnisse speichern

	sub  zer,R28									; Messwert von Zwischenergebnis abziehen
	ldi  R16, (1 << REFS0)|(1 << ADLAR)|(1 << MUX1)	; ADC auf linken Sensor einstellen
	out  ADMUX,R16
	sbi  PORTD,6									; Front LED einschalten
	mov  ml,zel										; Zwischenergebnis links als Messwert links speichern
	mov  mr,zer										; Zwischenergebis rechts als Messwert rechts speichern
	mov  diff,ml									; Differenz bilden (Messwert links - Messwert rechts)
	sub  diff,mr
	ldi  R18,drift									; Verschiebung zwischen rechten und linken Sensor korrigieren	
	add  diff,R18
	ldi  R16,1										; Zhler auf erste Messung setzen
	mov  adcc,R16
	clr  R16
	
	adci_4:  
	
	cbi ADCSRA,ADIE			; ADC Interrupt abschalten
	ldi R16,wartezeit		; Timer0 auf Wartezeit einstellen, damit LED vollstndig erleuchtet oder erloschen ist
	out TCNT0,R16			; Erst nach Ablauf der Wartezeit wird ber die ISR TIM0_OVF der ADC Interrupt wieder zugeschaltet und 
	ldi R16,(1 << TOIE0)	; ein neuer Messwert aufgenommen.
	out TIMSK,R16

	pop  R28				; Register wiederherstellen
	pop  R18
	pop  R16
	out  SREG,R16
	pop  R16

reti
