////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* APL / RECEIVE		              */
/* Version 1.0.0                      */
/* 2011-06-07                         */
/*                                    */
/**************************************/
////////////////////////////////////////



/***************   RECEIVE   **************/

RECEIVE:

	rcall RECEIVE_ENABLE	; speert das Senden und gibt das Empfangen am USART frei
	rcall STARTSEQUENCE		; wartet darauf das die StartSequence bermittelt wurde
	rcall RX_INFO			; empfngt die Zeichenkette und speichert diese im DRAM


ret



/***************   RECEIVE_ENABLE   ***************/

RECEIVE_ENABLE:

	cbi  UCSRB,TXEN		; Transmit disable
	cbi  UCSRB,TXCIE	; Transmit Complete Interrupt disable
	cbi  UCSRB,RXCIE	; Receive Complete Interrupt disable
	sbi  UCSRB,RXEN		; Receive enable
	sbi  UCSRA,RXC

ret



/***************   STARTSEQUENCE   ***************/

STARTSEQUENCE:
	
	push  ZH					; Adresse sichern
	push  ZL
	push  R16					; Register sichern

	ldi   ZH,high(text)			
	ldi   ZL,low(text)			; Adresse auf Array einstellen
	clr	  count					; Zhler rcksetzen

	STARTSEQUENCE_1:

	sbis  UCSRA,RXC				; Wenn Empfang noch nicht abgeschlossen ist warten
	rjmp  STARTSEQUENCE_1
	in    R16,UDR				; Empfangsregister auslesen
	st    Z+,R16				; und Wert im DRAM speichern und Z-Pointer an nchste Speicheradresse verschieben
	inc   count
	cpi   count,0x03			; Prfen ob bereits drei Zeichen eingelesen wurden
	brne  STARTSEQUENCE_1		; Wenn nicht weitere Zeichen empfangen
	rcall CHECKSTARTSEQUENCE	; Prfen ob die letzten drei empfangenen Zeichen die Startsequence (AS#) sind.	
	mov   R16,rx_seq
	cpi   R16,0xFF				; R0 ist Ergebnisregister der Funktion CHECKSTARTSEQUENCE
	breq  STARTSEQUENCE_3		; Wenn Startsequence empfangen wurde, dass warten auf die STartsequence beenden
	rcall SHIFTSEQUENCE			; Zeichen im Speicher nach links verschieben
	ldi   R16,0x01
	sub   ZL,R16
	brcc  STARTSEQUENCE_2
	dec	  ZH				

	STARTSEQUENCE_2:

	dec   count
	rjmp  STARTSEQUENCE_1

	STARTSEQUENCE_3:

	pop   R16					; Register wiederherstellen
	pop   ZL					; Adresse wiederherstellen
	pop   ZH

ret



/****************   CHECKSTARTSEQUENCE   ****************/

CHECKSTARTSEQUENCE:

	push  ZH					; Adresse sichern
	push  ZL
	push  R16					; Register sichern

	ldi   ZH,high(text)			; Startadresse einstellen
	ldi   ZL,low(text)
	clr   rx_seq				; Ergebnisregister lschen
	ld    R16,Z+				; erstes Zeichen laden und vergleichen
	cpi   R16,'A'              
	brne  CHECKSEQUENCE_1		
	ld    R16,Z+				; zweites Zeiche laden und vergleichen
	cpi   R16,'S'					
	brne  CHECKSEQUENCE_1
	ld    R16,Z+				; drittes Zeiche laden und vergleichen
	cpi   R16,'R'					
	brne  CHECKSEQUENCE_1
	ser   R16					; Wenn Startsequence empfangen wurde, dann Ergebnisregister setzen (R0 = 0xFF)
	mov   rx_seq,R16				


	CHECKSEQUENCE_1:

	pop   R16					; Register wiederherstellen
	pop   ZL					; Adresse wiederherstellen
	pop   ZH

ret



/****************   SHIFTSEQUENCE   ****************/

SHIFTSEQUENCE:

	push  YH			; Y-Pointer sichern
	push  YL
	push  ZH			; Z-Pointer sichern
	push  ZL
	push  R16			; Register sichern
	push  count

	ldi   count,0x02	; Zhler initialisieren
	ldi   YH,high(text)	; Y-Pointer intialisieren
	ldi   YL,low(text)
	ldi   ZH,high(text)	; Z-Pointer inialisieren
	ldi   ZL,low(text)
	ldi   R16,0x01
	add   ZL,R16
	brcc  SHIFTSEQUENCE_1
	inc   ZH

	SHIFTSEQUENCE_1:

	ld    R16,Z+			; Z-Wert auslesen und nach Y speichern
	st    Y+,R16
	dec   count				; Zhler dekrementieren
	brne  SHIFTSEQUENCE_1	; Wenn noch nicht 0 dann weiter schieben

	clr   R16				
	st    Y,R16	

	pop   count				; Register wiederherstellen
	pop   R16
	pop   ZL				; Z-Pointer wiederherstellen
	pop   ZH
	pop   YL				; Y-Pointer wiederherstellen
	pop   YH

ret



/**************   RX_INFO   ****************/

RX_INFO:

	push  ZH			; Z-Pointer sichern
	push  ZL
	push  R16			; Register sichern
	push  count

	ldi	  ZH,high(text)	; Startadresse einstellen
	ldi   ZL,low(text)
	clr   count			; Zhler auf 0 setzen
	rcall TIMEOUT_ON	; Timeout mittels Watchdog einschalten

	RX_INFO_1:

	WDR					; timeout reset

	RX_INFO_2:

	sbic  UCSRA,RXC		; Wenn Receive Complete nicht gesetzt ist, dann nchste Zeile bergehen
	rjmp  RX_INFO_3		; Wenn Receive Complet gesetzt ist dann weiter bei RX_INFO_3
	rjmp  RX_INFO_2		

	RX_INFO_3:

	in    R16,UDR		; empfangenes Zeichen auslesen
	st    Z+,R16		; und in DRAM speichern
	inc   count			; Zeichenzhler inkrementieren
	cpi   count,0x03      ; Prfen ob bereits mehr als zwei Zeichen empfangen wurden
	brcs  RX_INFO_1		; Wurden noch nicht mehr als 2 Zeichen empfangen, dann nchstes Zeichen empfangen
	rcall ENDSEQUENCE	; Wurden bereits mehr als drei Zeichen empfangen, dann prfen ob EndSequence empfangen wurde
	tst   rx_seq		; Ergebnisregister von ENDSEQUENCE ist R0, dieses prfen ob EndSequence empfangen wurde (0x00 nicht empfangen; 0xFF empfangen). 
	brne  RX_INFO_4		; Wenn ja, dann Empfang bei RX_INFO_4 beenden.
	cpi   count,250		; Wenn nicht, dann prfen ob bereits mehr als 250 Zeichen empfangen wurden
	brcs  RX_INFO_1		; Wenn noch nicht mehr als 250 Zeichen empfangen wurden, dann weiter mit empfang des nchsten Zeichens
	ldi   count,253		; Anzahl empfangerer Zeichen auf 250 setzen, wenn mehr als 250 Zeichen empfangen wurden

	RX_INFO_4:

	WDR					; timeout reset
	mov   anz,count		; Anzahl empfangener Zeichen in R10 speichern; R10 fungiert als Ergebnisregister fr RX_INFO
	ldi   R16,0x03		; Es werden 3 Zeichen abgezogen, damit spter nicht die Endsequence mit ausgesendet wird
	sub   anz,R16      	
	rcall TIMEOUT_OFF	; timeout (Watchdog) abschalten

	pop   count			; Register wiederherstellen
	pop   R16	
	pop   ZL			; Z-Pointer wiederherstellen
	pop   ZH

ret



/***************   TIMEOUT_OFF   **************/

TIMEOUT_OFF:

	push R16						; register sichern
	WDR								; reset WDT
	in   R16, WDTCR					; Write logical one to WDCE and WDE
	ori  R16, (1<<WDCE)|(1<<WDE)
	out  WDTCR, R16
	ldi  R16, (0<<WDE)				; Turn off WDT
	out  WDTCR, r16
	pop	 R16						; Register wiederherstellen

ret



/***************   TIMEOUT_ON   **************/

TIMEOUT_ON:

	push R16								; register sichern
	ldi  R16, (1<<WDCE)|(1<<WDE)
	out  WDTCR, R16
	ori  R16, (1<<WDE)|(1<<WDP2)|(1<<WDP1)	; Turn on WDT
	out  WDTCR, R16
	pop	 R16								; Register wiederherstellen

ret



/***************   ENDSEQUENCE   **************/

ENDSEQUENCE:

	push YH				; Y-Pointer sichern
	push YL	
	push ZH				; Z-Pointer sichern
	push ZL
	push R16			; Register sichern

	clr  rx_seq			; Ergebnisregister auf 0 fr nicht empfangene EndSequence setzen
	mov  YH,ZH			; Startadresse setzen
	mov  YL,ZL
	ldi  R16,0x03
	sub  YL,R16
	ld	 R16,Y+			; erstes Zeichen auslesen und prfen
	cpi  R16,'R'
	brne ENDSEQUENCE_1
	ld   R16,Y+			; zweites Zeichen auslesen und prfen
	cpi  R16,'S'		
	brne ENDSEQUENCE_1
	ld   R16,Y			; drittes Zeichen auslesen und prfen
	cpi  R16,'A'
	brne ENDSEQUENCE_1
	ser  R16			; wurde die EndSequence empfangen dann Ergebnisregister auf 0xFF (R0 = 0xFF) setzen.
	mov  rx_seq,R16
	
	ENDSEQUENCE_1:

	pop  R16	; Register wiederherstellen
	pop  ZL		; Z-Pointer wiederherstellen
	pop  ZH
	pop  YL		; Y-Pointer wiederherstellen
	pop  YH

ret




